@@ -26,6 +26,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #define __USE_MINGW_ANSI_STDIO 1
 #include <stdio.h>
 #include <stdlib.h>
@@ -57,7 +61,7 @@ int dfuload_do_upload(struct dfu_if *dif, int xfer_size,
 		int rc;
 		dfu_progress_bar("Upload", total_bytes, expected_size);
 		rc = dfu_upload(dif->dev_handle, dif->interface,
-		    xfer_size, transaction++, buf);
+				xfer_size, transaction++, buf);
 		if (rc < 0) {
 			warnx("\nError during upload (%s)",
 			      libusb_error_name(rc));
@@ -88,12 +92,13 @@ int dfuload_do_upload(struct dfu_if *dif, int xfer_size,
 		printf("\nFailed.\n");
 	else
 		printf("Received a total of %lli bytes\n", (long long) total_bytes);
+
 	if (expected_size != 0 && total_bytes != expected_size)
 		warnx("Unexpected number of bytes uploaded from device");
 	return ret;
 }
 
-off_t dfuload_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file *file)
+int dfuload_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file *file)
 {
 	off_t bytes_sent;
 	off_t expected_size;
@@ -120,7 +125,7 @@ off_t dfuload_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file *file
 			chunk_size = xfer_size;
 
 		ret = dfu_download(dif->dev_handle, dif->interface,
-		    chunk_size, transaction++, chunk_size ? buf : NULL);
+				   chunk_size, transaction++, chunk_size ? buf : NULL);
 		if (ret < 0) {
 			warnx("Error during download (%s)",
 			      libusb_error_name(ret));
@@ -147,9 +152,10 @@ off_t dfuload_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file *file
 				fprintf(stderr, "Poll timeout %i ms\n", dst.bwPollTimeout);
 
 		} while (1);
+
 		if (dst.bStatus != DFU_STATUS_OK) {
 			printf(" failed!\n");
-			printf("state(%u) = %s, status(%u) = %s\n", dst.bState,
+			printf("DFU state(%u) = %s, status(%u) = %s\n", dst.bState,
 				dfu_state_to_string(dst.bState), dst.bStatus,
 				dfu_status_to_string(dst.bStatus));
 			ret = -1;
@@ -159,8 +165,7 @@ off_t dfuload_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file *file
 	}
 
 	/* send one zero sized download request to signalize end */
-	ret = dfu_download(dif->dev_handle, dif->interface,
-	    0, transaction, NULL);
+	ret = dfu_download(dif->dev_handle, dif->interface, 0, transaction, NULL);
 	if (ret < 0) {
 		errx(EX_IOERR, "Error sending completion packet (%s)",
 		     libusb_error_name(ret));
@@ -180,7 +185,7 @@ get_status:
 		      libusb_error_name(ret));
 		goto out;
 	}
-	printf("state(%u) = %s, status(%u) = %s\n", dst.bState,
+	printf("DFU state(%u) = %s, status(%u) = %s\n", dst.bState,
 		dfu_state_to_string(dst.bState), dst.bStatus,
 		dfu_status_to_string(dst.bStatus));
 
@@ -209,5 +214,5 @@ get_status:
 	printf("Done!\n");
 
 out:
-	return bytes_sent;
+	return ret;
 }
