@@ -23,6 +23,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -33,9 +37,7 @@
 #include "dfu.h"
 #include "usb_dfu.h"
 #include "dfu_file.h"
-#include "dfu_load.h"
 #include "dfu_util.h"
-#include "dfuse.h"
 #include "quirks.h"
 
 /*
@@ -255,6 +257,8 @@ found_dfu:
 
 		for (intf_idx = 0; intf_idx < cfg->bNumInterfaces;
 		     intf_idx++) {
+			int multiple_alt;
+
 			if (match_iface_index > -1 && match_iface_index != intf_idx)
 				continue;
 
@@ -262,6 +266,8 @@ found_dfu:
 			if (!uif)
 				break;
 
+			multiple_alt = uif->num_altsetting > 0;
+
 			for (alt_idx = 0;
 			     alt_idx < uif->num_altsetting; alt_idx++) {
 				int dfu_mode;
@@ -312,8 +318,11 @@ found_dfu:
 				if (match_devnum >= 0 && match_devnum != libusb_get_device_address(dev))
 					continue;
 
-				if (libusb_open(dev, &devh)) {
-					warnx("Cannot open DFU device %04x:%04x", desc->idVendor, desc->idProduct);
+				ret = libusb_open(dev, &devh);
+				if (ret) {
+					warnx("Cannot open DFU device %04x:%04x found on devnum %i (%s)",
+					      desc->idVendor, desc->idProduct, libusb_get_device_address(dev),
+					      libusb_error_name(ret));
 					break;
 				}
 				if (intf->iInterface != 0)
@@ -375,6 +384,8 @@ found_dfu:
 					errx(EX_SOFTWARE, "Out of memory");
 				if (dfu_mode)
 					pdfu->flags |= DFU_IFF_DFU;
+				if (multiple_alt)
+					pdfu->flags |= DFU_IFF_ALT;
 				if (pdfu->quirks & QUIRK_FORCE_DFU11) {
 					pdfu->func_dfu.bcdDFUVersion =
 					  libusb_cpu_to_le16(0x0110);
@@ -430,7 +441,7 @@ void probe_devices(libusb_context *ctx)
 			continue;
 		probe_configuration(dev, &desc);
 	}
-	libusb_free_device_list(list, 0);
+	libusb_free_device_list(list, 1);
 }
 
 void disconnect_devices(void)
