@@ -2,8 +2,8 @@
 #define DFU_UTIL_H
 
 /* USB string descriptor should contain max 126 UTF-16 characters
- * but 253 would even accomodate any UTF-8 encoding */
-#define MAX_DESC_STR_LEN 253
+ * but 254 would even accommodate a UTF-8 encoding + NUL terminator */
+#define MAX_DESC_STR_LEN 254
 
 enum mode {
 	MODE_NONE,
@@ -15,8 +15,7 @@ enum mode {
 };
 
 extern struct dfu_if *dfu_root;
-extern int match_bus;
-extern int match_device;
+extern char *match_path;
 extern int match_vendor;
 extern int match_product;
 extern int match_vendor_dfu;
@@ -24,6 +23,7 @@ extern int match_product_dfu;
 extern int match_config_index;
 extern int match_iface_index;
 extern int match_iface_alt_index;
+extern int match_devnum;
 extern const char *match_iface_alt_name;
 extern const char *match_serial;
 extern const char *match_serial_dfu;
