@@ -22,6 +22,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -39,7 +43,6 @@
 
 extern int verbose;
 static unsigned int last_erased_page = 1; /* non-aligned value, won't match */
-static struct memsegment *mem_layout;
 static unsigned int dfuse_address = 0;
 static unsigned int dfuse_address_present = 0;
 static unsigned int dfuse_length = 0;
@@ -71,7 +74,7 @@ static void dfuse_parse_options(const char *options)
 			dfuse_address = number;
 			dfuse_address_present = 1;
 		} else {
-			errx(EX_IOERR, "Invalid dfuse address: %s", options);
+			errx(EX_USAGE, "Invalid dfuse address: %s", options);
 		}
 		options = endword;
 	}
@@ -116,7 +119,7 @@ static void dfuse_parse_options(const char *options)
 		if (end == endword) {
 			dfuse_length = number;
 		} else {
-			errx(EX_IOERR, "Invalid dfuse modifier: %s", options);
+			errx(EX_USAGE, "Invalid dfuse modifier: %s", options);
 		}
 		options = endword;
 	}
@@ -162,6 +165,9 @@ static int dfuse_download(struct dfu_if *dif, const unsigned short length,
 		 /* wLength       */	 length,
 					 DFU_TIMEOUT);
 	if (status < 0) {
+		/* Silently fail on leave request on some unpredictable devices */
+		if ((dif->quirks & QUIRK_DFUSE_LEAVE) && !length && !data && transaction == 2)
+			return status;
 		warnx("dfuse_download: libusb_control_transfer returned %d (%s)",
 		      status, libusb_error_name(status));
 	}
@@ -188,9 +194,9 @@ static int dfuse_special_command(struct dfu_if *dif, unsigned int address,
 		struct memsegment *segment;
 		int page_size;
 
-		segment = find_segment(mem_layout, address);
+		segment = find_segment(dif->mem_layout, address);
 		if (!segment || !(segment->memtype & DFUSE_ERASABLE)) {
-			errx(EX_IOERR, "Page at 0x%08x can not be erased",
+			errx(EX_USAGE, "Page at 0x%08x can not be erased",
 				address);
 		}
 		page_size = segment->pagesize;
@@ -214,7 +220,7 @@ static int dfuse_special_command(struct dfu_if *dif, unsigned int address,
 		buf[0] = 0x92;
 		length = 1;
 	} else {
-		errx(EX_IOERR, "Non-supported special command %d", command);
+		errx(EX_SOFTWARE, "Non-supported special command %d", command);
 	}
 	buf[1] = address & 0xff;
 	buf[2] = (address >> 8) & 0xff;
@@ -244,10 +250,10 @@ static int dfuse_special_command(struct dfu_if *dif, unsigned int address,
 		if (firstpoll) {
 			firstpoll = 0;
 			if (dst.bState != DFU_STATE_dfuDNBUSY) {
-				fprintf(stderr, "state(%u) = %s, status(%u) = %s\n", dst.bState,
+				fprintf(stderr, "DFU state(%u) = %s, status(%u) = %s\n", dst.bState,
 				       dfu_state_to_string(dst.bState), dst.bStatus,
 				       dfu_status_to_string(dst.bStatus));
-				errx(EX_IOERR, "Wrong state after command \"%s\" download",
+				errx(EX_PROTOCOL, "Wrong state after command \"%s\" download",
 				     dfuse_command_name[command]);
 			}
 			/* STM32F405 lies about mass erase timeout */
@@ -278,6 +284,7 @@ static int dfuse_special_command(struct dfu_if *dif, unsigned int address,
 	return ret;
 }
 
+/* returns number of bytes sent */
 static int dfuse_dnload_chunk(struct dfu_if *dif, unsigned char *data, int size,
 		       int transaction)
 {
@@ -308,8 +315,8 @@ static int dfuse_dnload_chunk(struct dfu_if *dif, unsigned char *data, int size,
 			printf("Transitioning to dfuMANIFEST state\n");
 
 	if (dst.bStatus != DFU_STATUS_OK) {
-		fprintf(stderr, " failed!\n");
-		fprintf(stderr, "state(%u) = %s, status(%u) = %s\n", dst.bState,
+		printf(" failed!\n");
+		fprintf(stderr, "DFU state(%u) = %s, status(%u) = %s\n", dst.bState,
 		       dfu_state_to_string(dst.bState), dst.bStatus,
 		       dfu_status_to_string(dst.bStatus));
 		return -1;
@@ -317,6 +324,22 @@ static int dfuse_dnload_chunk(struct dfu_if *dif, unsigned char *data, int size,
 	return bytes_sent;
 }
 
+static void dfuse_do_leave(struct dfu_if *dif)
+{
+	if (dfuse_address_present)
+		dfuse_special_command(dif, dfuse_address, SET_ADDRESS);
+	printf("Submitting leave request...\n");
+	if (dif->quirks & QUIRK_DFUSE_LEAVE) {
+		struct dfu_status dst;
+		/* The device might leave after this request, with or without a response */
+		dfuse_download(dif, 0, NULL, 2);
+		/* Or it might leave after this request, with or without a response */
+		dfu_get_status(dif, &dst);
+	} else {
+		dfuse_dnload_chunk(dif, NULL, 0, 2);
+	}
+}
+
 int dfuse_do_upload(struct dfu_if *dif, int xfer_size, int fd,
 		    const char *dfuse_options)
 {
@@ -333,7 +356,7 @@ int dfuse_do_upload(struct dfu_if *dif, int xfer_size, int fd,
 	if (dfuse_length)
 		upload_limit = dfuse_length;
 	if (dfuse_address_present) {
-		struct memsegment *segment;
+		struct memsegment *mem_layout, *segment;
 
 		mem_layout = parse_memory_layout((char *)dif->alt_name);
 		if (!mem_layout)
@@ -344,7 +367,7 @@ int dfuse_do_upload(struct dfu_if *dif, int xfer_size, int fd,
 		segment = find_segment(mem_layout, dfuse_address);
 		if (!dfuse_force &&
 		    (!segment || !(segment->memtype & DFUSE_READABLE)))
-			errx(EX_IOERR, "Page at 0x%08x is not readable",
+			errx(EX_USAGE, "Page at 0x%08x is not readable",
 				dfuse_address);
 
 		if (!upload_limit) {
@@ -393,7 +416,7 @@ int dfuse_do_upload(struct dfu_if *dif, int xfer_size, int fd,
 
 		if (rc < xfer_size || total_bytes >= upload_limit) {
 			/* last block, return successfully */
-			ret = total_bytes;
+			ret = 0;
 			break;
 		}
 		dfu_progress_bar("Upload", total_bytes, upload_limit);
@@ -402,11 +425,8 @@ int dfuse_do_upload(struct dfu_if *dif, int xfer_size, int fd,
 	dfu_progress_bar("Upload", total_bytes, total_bytes);
 
 	dfu_abort_to_idle(dif);
-	if (dfuse_leave) {
-		if (dfuse_address_present)
-			dfuse_special_command(dif, dfuse_address, SET_ADDRESS);
-		dfuse_dnload_chunk(dif, NULL, 0, 2); /* Zero-size */
-	}
+	if (dfuse_leave)
+		dfuse_do_leave(dif);
 
  out_free:
 	free(buf);
@@ -426,10 +446,10 @@ static int dfuse_dnload_element(struct dfu_if *dif, unsigned int dwElementAddres
 
 	/* Check at least that we can write to the last address */
 	segment =
-	    find_segment(mem_layout, dwElementAddress + dwElementSize - 1);
+	    find_segment(dif->mem_layout, dwElementAddress + dwElementSize - 1);
 	if (!dfuse_force &&
             (!segment || !(segment->memtype & DFUSE_WRITEABLE))) {
-		errx(EX_IOERR, "Last page at 0x%08x is not writeable",
+		errx(EX_USAGE, "Last page at 0x%08x is not writeable",
 			dwElementAddress + dwElementSize - 1);
 	}
 
@@ -443,10 +463,10 @@ static int dfuse_dnload_element(struct dfu_if *dif, unsigned int dwElementAddres
 		unsigned int address = dwElementAddress + p;
 		int chunk_size = xfer_size;
 
-		segment = find_segment(mem_layout, address);
+		segment = find_segment(dif->mem_layout, address);
 		if (!dfuse_force &&
 		    (!segment || !(segment->memtype & DFUSE_WRITEABLE))) {
-			errx(EX_IOERR, "Page at 0x%08x is not writeable",
+			errx(EX_USAGE, "Page at 0x%08x is not writeable",
 				address);
 		}
 		/* If the location is not in the memory map we skip erasing */
@@ -527,7 +547,7 @@ static void
 dfuse_memcpy(unsigned char *dst, unsigned char **src, int *rem, int size)
 {
 	if (size > *rem) {
-		errx(EX_IOERR, "Corrupt DfuSe file: "
+		errx(EX_NOINPUT, "Corrupt DfuSe file: "
 		    "Cannot read %d bytes from %d bytes", size, *rem);
 	}
 	if (dst != NULL)
@@ -556,13 +576,9 @@ static int dfuse_do_bin_dnload(struct dfu_if *dif, int xfer_size,
 
 	ret = dfuse_dnload_element(dif, dwElementAddress, dwElementSize, data,
 				   xfer_size);
-	if (ret != 0)
-		goto out_free;
+	if (ret == 0)
+		printf("File downloaded successfully\n");
 
-	printf("File downloaded successfully\n");
-	ret = dwElementSize;
-
- out_free:
 	return ret;
 }
 
@@ -577,6 +593,7 @@ static int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
 	int element;
 	int bTargets;
 	int bAlternateSetting;
+	struct dfu_if *adif;
 	int dwNbElements;
 	unsigned int dwElementAddress;
 	unsigned int dwElementSize;
@@ -591,31 +608,28 @@ static int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
         /* Must be larger than a minimal DfuSe header and suffix */
 	if (rem < (int)(sizeof(dfuprefix) +
 	    sizeof(targetprefix) + sizeof(elementheader))) {
-		errx(EX_SOFTWARE, "File too small for a DfuSe file");
+		errx(EX_DATAERR, "File too small for a DfuSe file");
         }
 
 	dfuse_memcpy(dfuprefix, &data, &rem, sizeof(dfuprefix));
 
 	if (strncmp((char *)dfuprefix, "DfuSe", 5)) {
-		errx(EX_IOERR, "No valid DfuSe signature");
+		errx(EX_DATAERR, "No valid DfuSe signature");
 		return -EINVAL;
 	}
 	if (dfuprefix[5] != 0x01) {
-		errx(EX_IOERR, "DFU format revision %i not supported",
+		errx(EX_DATAERR, "DFU format revision %i not supported",
 			dfuprefix[5]);
 		return -EINVAL;
 	}
 	bTargets = dfuprefix[10];
-	printf("file contains %i DFU images\n", bTargets);
-
-	printf("Please note that the next version of dfu-util will automatically\n"
-	       "set alternate interfaces based on the DfuSe file images!\n");
+	printf("File contains %i DFU images\n", bTargets);
 
 	for (image = 1; image <= bTargets; image++) {
-		printf("parsing DFU image %i\n", image);
+		printf("Parsing DFU image %i\n", image);
 		dfuse_memcpy(targetprefix, &data, &rem, sizeof(targetprefix));
 		if (strncmp((char *)targetprefix, "Target", 6)) {
-			errx(EX_IOERR, "No valid target signature");
+			errx(EX_DATAERR, "No valid target signature");
 			return -EINVAL;
 		}
 		bAlternateSetting = targetprefix[6];
@@ -624,17 +638,35 @@ static int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
 		else
 			printf("No target name\n");
 		dwNbElements = quad2uint((unsigned char *)targetprefix + 270);
-		printf("image for alternate setting %i, ", bAlternateSetting);
+		printf("Image for alternate setting %i, ", bAlternateSetting);
 		printf("(%i elements, ", dwNbElements);
 		printf("total size = %i)\n",
 		       quad2uint((unsigned char *)targetprefix + 266));
-		if (bAlternateSetting != dif->altsetting)
-			printf("Warning: Image does not match current alternate"
-			       " setting.\n"
-			       "Please rerun with the correct -a option setting"
-			       " to download this image!\n");
+
+		adif = dif;
+		while (adif) {
+			if (bAlternateSetting == adif->altsetting) {
+				adif->dev_handle = dif->dev_handle;
+				printf("Setting Alternate Interface #%d ...\n",
+				       adif->altsetting);
+				ret = libusb_set_interface_alt_setting(
+					  adif->dev_handle,
+					  adif->interface, adif->altsetting);
+				if (ret < 0) {
+					errx(EX_IOERR,
+					  "Cannot set alternate interface: %s",
+					  libusb_error_name(ret));
+				}
+				break;
+			}
+			adif = adif->next;
+		}
+		if (!adif)
+			warnx("No alternate setting %d (skipping elements)",
+			     bAlternateSetting);
+
 		for (element = 1; element <= dwNbElements; element++) {
-			printf("parsing element %i, ", element);
+			printf("Parsing element %i, ", element);
 			dfuse_memcpy(elementheader, &data, &rem, sizeof(elementheader));
 			dwElementAddress =
 			    quad2uint((unsigned char *)elementheader);
@@ -649,14 +681,13 @@ static int dfuse_do_dfuse_dnload(struct dfu_if *dif, int xfer_size,
 			}
 			/* sanity check */
 			if ((int)dwElementSize > rem)
-				errx(EX_SOFTWARE, "File too small for element size");
+				errx(EX_DATAERR, "File too small for element size");
 
-			if (bAlternateSetting == dif->altsetting) {
-				ret = dfuse_dnload_element(dif, dwElementAddress,
-				    dwElementSize, data, xfer_size);
-			} else {
+			if (adif)
+				ret = dfuse_dnload_element(adif, dwElementAddress,
+							   dwElementSize, data, xfer_size);
+			else
 				ret = 0;
-			}
 
 			/* advance read pointer */
 			dfuse_memcpy(NULL, &data, &rem, dwElementSize);
@@ -678,60 +709,89 @@ int dfuse_do_dnload(struct dfu_if *dif, int xfer_size, struct dfu_file *file,
 		    const char *dfuse_options)
 {
 	int ret;
+	struct dfu_if *adif;
 
 	if (dfuse_options)
 		dfuse_parse_options(dfuse_options);
-	mem_layout = parse_memory_layout((char *)dif->alt_name);
-	if (!mem_layout)
-		errx(EX_IOERR, "Failed to parse memory layout");
-	if (dif->quirks & QUIRK_DFUSE_LAYOUT)
-		fixup_dfuse_layout(dif, &mem_layout);
+
+	adif = dif;
+	while (adif) {
+		adif->mem_layout = parse_memory_layout((char *)adif->alt_name);
+		if (!adif->mem_layout)
+			errx(EX_IOERR,
+			     "Failed to parse memory layout for alternate interface %i",
+			     adif->altsetting);
+		if (adif->quirks & QUIRK_DFUSE_LAYOUT)
+			fixup_dfuse_layout(adif, &(adif->mem_layout));
+		adif = adif->next;
+	}
 
 	if (dfuse_unprotect) {
 		if (!dfuse_force) {
-			errx(EX_IOERR, "The read unprotect command "
+			errx(EX_USAGE, "The read unprotect command "
 				"will erase the flash memory"
 				"and can only be used with force\n");
 		}
-		dfuse_special_command(dif, 0, READ_UNPROTECT);
+		ret = dfuse_special_command(dif, 0, READ_UNPROTECT);
 		printf("Device disconnects, erases flash and resets now\n");
-		exit(0);
+		return ret;
 	}
 	if (dfuse_mass_erase) {
 		if (!dfuse_force) {
-			errx(EX_IOERR, "The mass erase command "
+			errx(EX_USAGE, "The mass erase command "
 				"can only be used with force");
 		}
 		printf("Performing mass erase, this can take a moment\n");
-		dfuse_special_command(dif, 0, MASS_ERASE);
+		ret = dfuse_special_command(dif, 0, MASS_ERASE);
 	}
 	if (!file->name) {
 		printf("DfuSe command mode\n");
 		ret = 0;
 	} else if (dfuse_address_present) {
 		if (file->bcdDFU == 0x11a) {
-			errx(EX_IOERR, "This is a DfuSe file, not "
+			errx(EX_USAGE, "This is a DfuSe file, not "
 				"meant for raw download");
 		}
 		ret = dfuse_do_bin_dnload(dif, xfer_size, file, dfuse_address);
 	} else {
 		if (file->bcdDFU != 0x11a) {
 			warnx("Only DfuSe file version 1.1a is supported");
-			errx(EX_IOERR, "(for raw binary download, use the "
+			errx(EX_USAGE, "(for raw binary download, use the "
 			     "--dfuse-address option)");
 		}
 		ret = dfuse_do_dfuse_dnload(dif, xfer_size, file);
 	}
-	free_segment_list(mem_layout);
+
+	adif = dif;
+	while (adif) {
+		free_segment_list(adif->mem_layout);
+		adif = adif->next;
+	}
 
 	if (!dfuse_will_reset) {
 		dfu_abort_to_idle(dif);
 	}
 
-	if (dfuse_leave) {
-		if (dfuse_address_present)
-			dfuse_special_command(dif, dfuse_address, SET_ADDRESS);
-		dfuse_dnload_chunk(dif, NULL, 0, 2); /* Zero-size */
-	}
+	if (dfuse_leave)
+		dfuse_do_leave(dif);
+
 	return ret;
 }
+
+/* Check if we have one interface, possibly multiple alternate interfaces */
+int dfuse_multiple_alt(struct dfu_if *dfu_root)
+{
+	libusb_device *dev = dfu_root->dev;
+	uint8_t configuration = dfu_root->configuration;
+	uint8_t interface = dfu_root->interface;
+	struct dfu_if *dif = dfu_root->next;
+
+	while (dif) {
+		if (dev != dif->dev ||
+		    configuration != dif->configuration ||
+		    interface != dif->interface)
+			return 0;
+		dif = dif->next;
+	}
+	return 1;
+}
