@@ -4,7 +4,7 @@
  * Copyright 2007-2008 by OpenMoko, Inc.
  * Copyright 2010-2012 Stefan Schmidt
  * Copyright 2013-2014 Hans Petter Selasky <hps@bitfrost.no>
- * Copyright 2010-2020 Tormod Volden
+ * Copyright 2010-2021 Tormod Volden
  *
  * Originally written by Harald Welte <laforge@openmoko.org>
  *
@@ -25,6 +25,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -149,11 +153,11 @@ static int parse_number(char *str, char *nmb)
 
 	if ((errno == ERANGE && (val == LONG_MAX || val == LONG_MIN))
 		|| (errno != 0 && val == 0) || (*endptr != '\0')) {
-		errx(EX_SOFTWARE, "Something went wrong with the argument of --%s\n", str);
+		errx(EX_USAGE, "Something went wrong with the argument of --%s\n", str);
 	}
 
 	if (endptr == nmb) {
-		errx(EX_SOFTWARE, "No digits were found from the argument of --%s\n", str);
+		errx(EX_USAGE, "No digits were found from the argument of --%s\n", str);
 	}
 
 	return (int)val;
@@ -195,14 +199,13 @@ static void help(void)
 		"\t\tforce\t\tYou really know what you are doing!\n"
 		"\t\t<length>\tLength of firmware to upload from device\n"
 		);
-	exit(EX_USAGE);
 }
 
 static void print_version(void)
 {
 	printf(PACKAGE_STRING "\n\n");
 	printf("Copyright 2005-2009 Weston Schmidt, Harald Welte and OpenMoko Inc.\n"
-	       "Copyright 2010-2020 Tormod Volden and Stefan Schmidt\n"
+	       "Copyright 2010-2021 Tormod Volden and Stefan Schmidt\n"
 	       "This program is Free Software and has ABSOLUTELY NO WARRANTY\n"
 	       "Please report bugs to " PACKAGE_BUGREPORT "\n\n");
 }
@@ -268,6 +271,7 @@ int main(int argc, char **argv)
 		switch (c) {
 		case 'h':
 			help();
+			exit(EX_OK);
 			break;
 		case 'V':
 			mode = MODE_VERSION;
@@ -341,17 +345,19 @@ int main(int argc, char **argv)
 			break;
 		default:
 			help();
+			exit(EX_USAGE);
 			break;
 		}
 	}
 	if (optind != argc) {
 		fprintf(stderr, "Error: Unexpected argument: %s\n\n", argv[optind]);
 		help();
+		exit(EX_USAGE);
 	}
 
 	print_version();
 	if (mode == MODE_VERSION) {
-		exit(0);
+		exit(EX_OK);
 	}
 
 #if defined(LIBUSB_API_VERSION) || defined(LIBUSBX_API_VERSION)
@@ -368,6 +374,7 @@ int main(int argc, char **argv)
 	if (mode == MODE_NONE && !dfuse_options) {
 		fprintf(stderr, "You need to specify one of -D or -U\n");
 		help();
+		exit(EX_USAGE);
 	}
 
 	if (match_config_index == 0) {
@@ -414,7 +421,9 @@ probe:
 
 	if (mode == MODE_LIST) {
 		list_dfu_interfaces();
-		exit(0);
+		disconnect_devices();
+		libusb_exit(ctx);
+		return EX_OK;
 	}
 
 	if (dfu_root == NULL) {
@@ -422,8 +431,12 @@ probe:
 			milli_sleep(20);
 			goto probe;
 		} else {
-			errx(EX_IOERR, "No DFU capable USB device available");
+			warnx("No DFU capable USB device available");
+			libusb_exit(ctx);
+			return EX_IOERR;
 		}
+	} else if (file.bcdDFU == 0x11a && dfuse_multiple_alt(dfu_root)) {
+		printf("Multiple alternate interfaces for DfuSe file\n");
 	} else if (dfu_root->next != NULL) {
 		/* We cannot safely support more than one DFU capable device
 		 * with same vendor/product ID, since during DFU we need to do
@@ -441,11 +454,30 @@ probe:
 	if (ret || !dfu_root->dev_handle)
 		errx(EX_IOERR, "Cannot open device: %s", libusb_error_name(ret));
 
-	printf("ID %04x:%04x\n", dfu_root->vendor, dfu_root->product);
+	printf("Device ID %04x:%04x\n", dfu_root->vendor, dfu_root->product);
 
-	printf("Run-time device DFU version %04x\n",
+	/* If first interface is DFU it is likely not proper run-time */
+	if (dfu_root->interface > 0)
+		printf("Run-Time device");
+	else
+		printf("Device");
+	printf(" DFU version %04x\n",
 	       libusb_le16_to_cpu(dfu_root->func_dfu.bcdDFUVersion));
 
+	if (verbose) {
+		printf("DFU attributes: (0x%02x)", dfu_root->func_dfu.bmAttributes);
+		if (dfu_root->func_dfu.bmAttributes & USB_DFU_CAN_DOWNLOAD)
+			printf(" bitCanDnload");
+		if (dfu_root->func_dfu.bmAttributes & USB_DFU_CAN_UPLOAD)
+			printf(" bitCanUpload");
+		if (dfu_root->func_dfu.bmAttributes & USB_DFU_MANIFEST_TOL)
+			printf(" bitManifestationTolerant");
+		if (dfu_root->func_dfu.bmAttributes & USB_DFU_WILL_DETACH)
+			printf(" bitWillDetach");
+		printf("\n");
+		printf("Detach timeout %d ms\n", libusb_le16_to_cpu(dfu_root->func_dfu.wDetachTimeOut));
+	}
+
 	/* Transition from run-Time mode to DFU mode */
 	if (!(dfu_root->flags & DFU_IFF_DFU)) {
 		int err;
@@ -457,20 +489,26 @@ probe:
 		runtime_vendor = dfu_root->vendor;
 		runtime_product = dfu_root->product;
 
-		printf("Claiming USB DFU Runtime Interface...\n");
+		printf("Claiming USB DFU (Run-Time) Interface...\n");
 		ret = libusb_claim_interface(dfu_root->dev_handle, dfu_root->interface);
 		if (ret < 0) {
 			errx(EX_IOERR, "Cannot claim interface %d: %s",
 				dfu_root->interface, libusb_error_name(ret));
 		}
 
-		ret = libusb_set_interface_alt_setting(dfu_root->dev_handle, dfu_root->interface, 0);
-		if (ret < 0) {
-			errx(EX_IOERR, "Cannot set alt interface zero: %s", libusb_error_name(ret));
+		/* Needed for some devices where the DFU interface is not the first,
+		 * and should also be safe if there are multiple alt settings.
+		 * Otherwise skip the request since it might not be supported
+		 * by the device and the USB stack may or may not recover */
+		if (dfu_root->interface > 0 || dfu_root->flags & DFU_IFF_ALT) {
+			printf("Setting Alternate Interface zero...\n");
+			ret = libusb_set_interface_alt_setting(dfu_root->dev_handle, dfu_root->interface, 0);
+			if (ret < 0) {
+				errx(EX_IOERR, "Cannot set alternate interface zero: %s", libusb_error_name(ret));
+			}
 		}
 
-		printf("Determining device status: ");
-
+		printf("Determining device status...\n");
 		err = dfu_get_status(dfu_root, &status);
 		if (err == LIBUSB_ERROR_PIPE) {
 			printf("Device does not implement get_status, assuming appIDLE\n");
@@ -479,17 +517,18 @@ probe:
 			status.bState  = DFU_STATE_appIDLE;
 			status.iString = 0;
 		} else if (err < 0) {
-			errx(EX_IOERR, "error get_status");
+			errx(EX_IOERR, "error get_status: %s", libusb_error_name(err));
 		} else {
-			printf("state = %s, status = %d\n",
-			       dfu_state_to_string(status.bState), status.bStatus);
+			printf("DFU state(%u) = %s, status(%u) = %s\n", status.bState,
+			       dfu_state_to_string(status.bState), status.bStatus,
+			       dfu_status_to_string(status.bStatus));
 		}
 		milli_sleep(status.bwPollTimeout);
 
 		switch (status.bState) {
 		case DFU_STATE_appIDLE:
 		case DFU_STATE_appDETACH:
-			printf("Device really in Runtime Mode, send DFU "
+			printf("Device really in Run-Time Mode, send DFU "
 			       "detach request...\n");
 			if (dfu_detach(dfu_root->dev_handle,
 				       dfu_root->interface, 1000) < 0) {
@@ -513,7 +552,8 @@ probe:
 			}
 			/* fall through */
 		default:
-			warnx("WARNING: Runtime device already in DFU state ?!?");
+			warnx("WARNING: Device already in DFU mode? (bState=%d %s)",
+			      status.bState, dfu_state_to_string(status.bState));
 			libusb_release_interface(dfu_root->dev_handle,
 			    dfu_root->interface);
 			goto dfustate;
@@ -523,14 +563,14 @@ probe:
 		libusb_close(dfu_root->dev_handle);
 		dfu_root->dev_handle = NULL;
 
-		if (mode == MODE_DETACH) {
-			libusb_exit(ctx);
-			exit(0);
-		}
-
 		/* keeping handles open might prevent re-enumeration */
 		disconnect_devices();
 
+		if (mode == MODE_DETACH) {
+			libusb_exit(ctx);
+			return EX_OK;
+		}
+
 		milli_sleep(detach_delay * 1000);
 
 		/* Change match vendor and product to impossible values to force
@@ -549,7 +589,7 @@ probe:
 
 		/* Check for DFU mode device */
 		if (!(dfu_root->flags | DFU_IFF_DFU))
-			errx(EX_SOFTWARE, "Device is not in DFU mode");
+			errx(EX_PROTOCOL, "Device is not in DFU mode");
 
 		printf("Opening DFU USB Device...\n");
 		ret = libusb_open(dfu_root->dev, &dfu_root->dev_handle);
@@ -579,30 +619,33 @@ dfustate:
 		errx(EX_IOERR, "Cannot claim interface - %s", libusb_error_name(ret));
 	}
 
-	printf("Setting Alternate Setting #%d ...\n", dfu_root->altsetting);
-	ret = libusb_set_interface_alt_setting(dfu_root->dev_handle, dfu_root->interface, dfu_root->altsetting);
-	if (ret < 0) {
-		errx(EX_IOERR, "Cannot set alternate interface: %s", libusb_error_name(ret));
+	if (dfu_root->flags & DFU_IFF_ALT) {
+		printf("Setting Alternate Interface #%d ...\n", dfu_root->altsetting);
+		ret = libusb_set_interface_alt_setting(dfu_root->dev_handle, dfu_root->interface, dfu_root->altsetting);
+		if (ret < 0) {
+			errx(EX_IOERR, "Cannot set alternate interface: %s", libusb_error_name(ret));
+		}
 	}
 
 status_again:
-	printf("Determining device status: ");
+	printf("Determining device status...\n");
 	ret = dfu_get_status(dfu_root, &status );
 	if (ret < 0) {
 		errx(EX_IOERR, "error get_status: %s", libusb_error_name(ret));
 	}
-	printf("state = %s, status = %d\n",
-	       dfu_state_to_string(status.bState), status.bStatus);
+	printf("DFU state(%u) = %s, status(%u) = %s\n", status.bState,
+	       dfu_state_to_string(status.bState), status.bStatus,
+	       dfu_status_to_string(status.bStatus));
 
 	milli_sleep(status.bwPollTimeout);
 
 	switch (status.bState) {
 	case DFU_STATE_appIDLE:
 	case DFU_STATE_appDETACH:
-		errx(EX_IOERR, "Device still in Runtime Mode!");
+		errx(EX_PROTOCOL, "Device still in Run-Time Mode!");
 		break;
 	case DFU_STATE_dfuERROR:
-		printf("dfuERROR, clearing status\n");
+		printf("Clearing status\n");
 		if (dfu_clear_status(dfu_root->dev_handle, dfu_root->interface) < 0) {
 			errx(EX_IOERR, "error clear_status");
 		}
@@ -610,15 +653,13 @@ status_again:
 		break;
 	case DFU_STATE_dfuDNLOAD_IDLE:
 	case DFU_STATE_dfuUPLOAD_IDLE:
-		printf("aborting previous incomplete transfer\n");
+		printf("Aborting previous incomplete transfer\n");
 		if (dfu_abort(dfu_root->dev_handle, dfu_root->interface) < 0) {
 			errx(EX_IOERR, "can't send DFU_ABORT");
 		}
 		goto status_again;
 		break;
 	case DFU_STATE_dfuIDLE:
-		printf("dfuIDLE, continuing\n");
-		break;
 	default:
 		break;
 	}
@@ -632,7 +673,7 @@ status_again:
 		if (dfu_get_status(dfu_root, &status) < 0)
 			errx(EX_IOERR, "USB communication error");
 		if (DFU_STATUS_OK != status.bStatus)
-			errx(EX_SOFTWARE, "Status is not OK: %d", status.bStatus);
+			errx(EX_PROTOCOL, "Status is not OK: %d", status.bStatus);
 
 		milli_sleep(status.bwPollTimeout);
 	}
@@ -655,19 +696,16 @@ status_again:
 			printf("Warning: Overriding device-reported transfer size\n");
 	} else {
 		if (!transfer_size)
-			errx(EX_IOERR, "Transfer size must be specified");
+			errx(EX_USAGE, "Transfer size must be specified");
 	}
 
-#ifdef HAVE_GETPAGESIZE
-/* autotools lie when cross-compiling for Windows using mingw32/64 */
-#ifndef __MINGW32__
-	/* limitation of Linux usbdevio */
-	if ((int)transfer_size > getpagesize()) {
-		transfer_size = getpagesize();
+#ifdef __linux__
+	/* limited to 4k in libusb Linux backend */
+	if ((int)transfer_size > 4096) {
+		transfer_size = 4096;
 		printf("Limited transfer size to %i\n", transfer_size);
 	}
-#endif /* __MINGW32__ */
-#endif /* HAVE_GETPAGESIZE */
+#endif /* __linux__ */
 
 	if (transfer_size < dfu_root->bMaxPacketSize0) {
 		transfer_size = dfu_root->bMaxPacketSize0;
@@ -678,20 +716,22 @@ status_again:
 	case MODE_UPLOAD:
 		/* open for "exclusive" writing */
 		fd = open(file.name, O_WRONLY | O_BINARY | O_CREAT | O_EXCL | O_TRUNC, 0666);
-		if (fd < 0)
-			err(EX_IOERR, "Cannot open file %s for writing", file.name);
+		if (fd < 0) {
+			warn("Cannot open file %s for writing", file.name);
+			ret = EX_CANTCREAT;
+			break;
+		}
 
 		if (dfuse_device || dfuse_options) {
-		    if (dfuse_do_upload(dfu_root, transfer_size, fd,
-					dfuse_options) < 0)
-			exit(1);
+		    ret = dfuse_do_upload(dfu_root, transfer_size, fd, dfuse_options);
 		} else {
-		    if (dfuload_do_upload(dfu_root, transfer_size,
-			expected_size, fd) < 0) {
-			exit(1);
-		    }
+		    ret = dfuload_do_upload(dfu_root, transfer_size, expected_size, fd);
 		}
 		close(fd);
+		if (ret < 0)
+			ret = EX_IOERR;
+		else
+			ret = EX_OK;
 		break;
 
 	case MODE_DOWNLOAD:
@@ -699,47 +739,55 @@ status_again:
 		     (file.idProduct != 0xffff && file.idProduct != runtime_product)) &&
 		    ((file.idVendor  != 0xffff && file.idVendor  != dfu_root->vendor) ||
 		     (file.idProduct != 0xffff && file.idProduct != dfu_root->product))) {
-			errx(EX_IOERR, "Error: File ID %04x:%04x does "
+			errx(EX_USAGE, "Error: File ID %04x:%04x does "
 				"not match device (%04x:%04x or %04x:%04x)",
 				file.idVendor, file.idProduct,
 				runtime_vendor, runtime_product,
 				dfu_root->vendor, dfu_root->product);
 		}
 		if (dfuse_device || dfuse_options || file.bcdDFU == 0x11a) {
-		        if (dfuse_do_dnload(dfu_root, transfer_size, &file,
-							dfuse_options) < 0)
-				exit(1);
+			ret = dfuse_do_dnload(dfu_root, transfer_size, &file, dfuse_options);
 		} else {
-			if (dfuload_do_dnload(dfu_root, transfer_size, &file) < 0)
-				exit(1);
+			ret = dfuload_do_dnload(dfu_root, transfer_size, &file);
 	 	}
+		if (ret < 0)
+			ret = EX_IOERR;
+		else
+			ret = EX_OK;
 		break;
 	case MODE_DETACH:
-		if (dfu_detach(dfu_root->dev_handle, dfu_root->interface, 1000) < 0) {
+		ret = dfu_detach(dfu_root->dev_handle, dfu_root->interface, 1000);
+		if (ret < 0) {
 			warnx("can't detach");
+			/* allow combination with final_reset */
+			ret = 0;
 		}
 		break;
 	default:
-		errx(EX_IOERR, "Unsupported mode: %u", mode);
+		warnx("Unsupported mode: %u", mode);
+		ret = EX_SOFTWARE;
 		break;
 	}
 
-	if (final_reset) {
-		if (dfu_detach(dfu_root->dev_handle, dfu_root->interface, 1000) < 0) {
+	if (!ret && final_reset) {
+		ret = dfu_detach(dfu_root->dev_handle, dfu_root->interface, 1000);
+		if (ret < 0) {
 			/* Even if detach failed, just carry on to leave the
                            device in a known state */
 			warnx("can't detach");
 		}
-		printf("Resetting USB to switch back to runtime mode\n");
+		printf("Resetting USB to switch back to Run-Time mode\n");
 		ret = libusb_reset_device(dfu_root->dev_handle);
 		if (ret < 0 && ret != LIBUSB_ERROR_NOT_FOUND) {
-			errx(EX_IOERR, "error resetting after download: %s", libusb_error_name(ret));
+			warnx("error resetting after download: %s", libusb_error_name(ret));
+			ret = EX_IOERR;
 		}
 	}
 
 	libusb_close(dfu_root->dev_handle);
 	dfu_root->dev_handle = NULL;
-	libusb_exit(ctx);
 
-	return (0);
+	disconnect_devices();
+	libusb_exit(ctx);
+	return ret;
 }
