@@ -2,12 +2,10 @@
 #ifndef PORTABLE_H
 #define PORTABLE_H
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#else
+#ifndef HAVE_CONFIG_H
 # define PACKAGE "dfu-util"
-# define PACKAGE_VERSION "0.10-msvc"
-# define PACKAGE_STRING "dfu-util 0.10-msvc"
+# define PACKAGE_VERSION "0.11-msvc"
+# define PACKAGE_STRING "dfu-util 0.11-msvc"
 # define PACKAGE_BUGREPORT "http://sourceforge.net/p/dfu-util/tickets/"
 # include <io.h>
 /* FIXME if off_t is a typedef it is not a define */
@@ -60,8 +58,12 @@
 #else
 # define EX_OK		0	/* successful termination */
 # define EX_USAGE	64	/* command line usage error */
+# define EX_DATAERR	65
+# define EX_NOINPUT	66
 # define EX_SOFTWARE	70	/* internal software error */
+# define EX_CANTCREAT	73	/* input/output error */
 # define EX_IOERR	74	/* input/output error */
+# define EX_PROTOCOL	76	/* input/output error */
 #endif /* HAVE_SYSEXITS_H */
 
 #ifndef O_BINARY
